from functools import wraps
from django.shortcuts import redirect
from django.urls import reverse
from django.conf import settings

def syauth_login_required(view_func):
    """
    Decorator for views that checks that the user is logged in,
    redirecting to the Syauth login page if necessary.
    """
    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        if request.user.is_authenticated:
            return view_func(request, *args, **kwargs)
        return redirect('django_syauth:login')
    return _wrapped_view



import hashlib
import base64
import os
import requests
import jwt
from django.conf import settings
from urllib.parse import urlencode

def get_config():
    """Retrieve Syauth configuration from Django settings."""
    config = getattr(settings, 'SYAUTH', {})
    required_keys = ['API_URL', 'CLIENT_ID', 'REDIRECT_URI']
    
    missing = [key for key in required_keys if key not in config]
    if missing:
        raise ValueError(f"Missing required SYAUTH settings: {', '.join(missing)}")
        
    return config

def generate_pkce_pair():
    """Generate a PKCE code verifier and code challenge."""
    # Generate a random 32-byte state
    code_verifier = base64.urlsafe_b64encode(os.urandom(32)).decode('utf-8').rstrip('=')
    
    # Create code challenge
    m = hashlib.sha256()
    m.update(code_verifier.encode('utf-8'))
    code_challenge = base64.urlsafe_b64encode(m.digest()).decode('utf-8').rstrip('=')
    
    return code_verifier, code_challenge

def get_authorization_url(state, code_challenge):
    """Build the authorization URL."""
    config = get_config()
    params = {
        'client_id': config['CLIENT_ID'],
        'response_type': 'code',
        'redirect_uri': config['REDIRECT_URI'],
        'state': state,
        'code_challenge': code_challenge,
        'code_challenge_method': 'S256',
        'scope': ' '.join(config.get('SCOPES', ['openid', 'profile', 'email'])),
    }
    
    # Add API Key if present (required for some environments)
    if 'API_KEY' in config:
        params['api_key'] = config['API_KEY']
    
    # Construct base URL correctly handling trailing slashes
    base_url = config['API_URL'].rstrip('/')
    return f"{base_url}/oauth/authorize/?{urlencode(params)}"

def exchange_code_for_token(code, code_verifier):
    """Exchange authorization code for tokens."""
    config = get_config()
    
    token_url = f"{config['API_URL'].rstrip('/')}/oauth/token/"
    payload = {
        'grant_type': 'authorization_code',
        'client_id': config['CLIENT_ID'],
        'client_secret': config.get('CLIENT_SECRET', ''),
        'code': code,
        'redirect_uri': config['REDIRECT_URI'],
        'code_verifier': code_verifier,
    }
    
    response = requests.post(token_url, data=payload)
    response.raise_for_status()
    return response.json()

def decode_id_token(id_token):
    """Decode and validate the ID token (simple decoding for now)."""
    # In production, this should verify the signature using JWKS
    # For now, we decode without verification to get user info
    return jwt.decode(id_token, options={"verify_signature": False})



import jwt
from django.utils.deprecation import MiddlewareMixin
from django.contrib.auth import get_user_model
from django.http import JsonResponse
from .utils import get_config

class SyauthRemoteUserMiddleware(MiddlewareMixin):
    def process_request(self, request):
        # Check for Authorization header
        auth_header = request.META.get('HTTP_AUTHORIZATION', '')
        if not auth_header.startswith('Bearer '):
            return

        token = auth_header.split(' ')[1]
        try:
            # Decode token (verify signature in production)
            payload = jwt.decode(token, options={"verify_signature": False})
            
            user_id = payload.get('sub')
            email = payload.get('email')
            
            if user_id:
                User = get_user_model()
                # Get or create user (stateless auth)
                try:
                    user = User.objects.get(username=user_id)
                except User.DoesNotExist:
                    if email:
                        user = User(username=user_id, email=email)
                        user.set_unusable_password()
                        user.save()
                    else:
                        return
                
                request.user = user

        except jwt.ExpiredSignatureError:
            return JsonResponse({'error': 'Token expired'}, status=401)
        except jwt.InvalidTokenError:
            return JsonResponse({'error': 'Invalid token'}, status=401)



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model

class SyauthBackend(ModelBackend):
    def authenticate(self, request, token_info=None, **kwargs):
        """
        Authenticates a user based on the token information from Syauth.
        expected token_info: dict containing 'sub', 'email', 'given_name', 'family_name'
        """
        if not token_info:
            return None
            
        User = get_user_model()
        user_id = token_info.get('sub')
        email = token_info.get('email')
        
        if not user_id or not email:
            return None
            
        # Try to find user by username (using syauth_id) or email
        try:
            user = User.objects.get(username=user_id)
        except User.DoesNotExist:
            try:
                user = User.objects.get(email=email)
            except User.DoesNotExist:
                # Create a new user
                user = User(username=user_id, email=email)
                user.set_unusable_password()
        
        # Update user details
        user.first_name = token_info.get('given_name', '')
        user.last_name = token_info.get('family_name', '')
        user.save()
        
        return user

    def get_user(self, user_id):
        User = get_user_model()
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.urls import path
from . import views

app_name = 'django_syauth'

urlpatterns = [
    path('login/', views.LoginView.as_view(), name='login'),
    path('callback/', views.CallbackView.as_view(), name='callback'),
    path('logout/', views.LogoutView.as_view(), name='logout'),
]



import os
import secrets
from django.shortcuts import redirect
from django.conf import settings
from django.views import View
from django.contrib.auth import login, logout
from django.http import HttpResponseBadRequest, HttpResponseRedirect
from django.urls import reverse
from .utils import generate_pkce_pair, get_authorization_url, exchange_code_for_token, decode_id_token, get_config
from .backend import SyauthBackend

class LoginView(View):
    def get(self, request):
        code_verifier, code_challenge = generate_pkce_pair()
        state = secrets.token_urlsafe(16)
        
        # Store verifier and state in session
        request.session['syauth_code_verifier'] = code_verifier
        request.session['syauth_state'] = state
        
        # Redirect to Syauth
        auth_url = get_authorization_url(state, code_challenge)
        return redirect(auth_url)

class CallbackView(View):
    def get(self, request):
        error = request.GET.get('error')
        if error:
            return HttpResponseBadRequest(f"Login failed: {error}")
            
        code = request.GET.get('code')
        state = request.GET.get('state')
        stored_state = request.session.get('syauth_state')
        code_verifier = request.session.get('syauth_code_verifier')
        
        if not code or not state or not stored_state or not code_verifier:
            return HttpResponseBadRequest("Missing required parameters or session expired")
            
        if state != stored_state:
            return HttpResponseBadRequest("Invalid state parameter")
            
        try:
            tokens = exchange_code_for_token(code, code_verifier)
            id_token = tokens.get('id_token')
            
            if not id_token:
                return HttpResponseBadRequest("No ID token received")
                
            user_info = decode_id_token(id_token)
            
            # Authenticate user
            backend = SyauthBackend()
            user = backend.authenticate(request, token_info=user_info)
            
            if user:
                login(request, user, backend='django_syauth.backend.SyauthBackend')
                
                # Store access token in session for API calls
                access_token = tokens.get('access_token')
                if access_token:
                    request.session['syauth_access_token'] = access_token
                
                # Cleanup session
                del request.session['syauth_code_verifier']
                del request.session['syauth_state']
                
                return redirect(settings.LOGIN_REDIRECT_URL)
            else:
                return HttpResponseBadRequest("Authentication failed")
                
        except Exception as e:
            return HttpResponseBadRequest(f"Authentication error: {str(e)}")

class LogoutView(View):
    def get(self, request):
        logout(request)
        config = get_config()
        # Optional: Redirect to Syauth logout if supported, otherwise home
        # For now, just redirect to home
        return redirect('/')
